/*************************************************************************
*                                                                        *
*   Module: meta.h                                                       *
*                                                                        *
*   Definitions, data structures types and imported headers              *
*                                                                        *
*************************************************************************/     

/* Required libraries */
#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <malloc.h>  
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <float.h>
#include <math.h>
#include <sys/stat.h>

/*************************************************************************
(A). DEFINITIONS
*************************************************************************/

#define MAXSITES 1500
#define MAXCOLISIONS 150
#define MAXPAIRS 100
#define MAXFACTORS 1000

#define MAXLINE 1000             /* Maximum chars per input line         */
#define MAXSTRING 100
#define FILENAMELENGTH 100 

#define MINDIST 1                /* Minimum distance between two factors */

#define ALPHA_DEFAULT 1
#define LAMBDA_DEFAULT 1
#define MU_DEFAULT 1

#define VERSION   "meta_v1.1"    /* The name of the game                 */

#define PERCENT 100
#define MINUTE 60                    
#define MEGABYTE 1048576
#define MAXTIMES 100

#define TOKEN "$"
#define NTOKENS 1
#define NOSCORE 1.000
#define INFI 999999999

#define BEGIN -1
#define GHOST -1

/* Macros (functions) */
#define MIN(a,b) (a<b)?a:b;
#define MAX(a,b) (a>b)?a:b;

/*************************************************************************
(B). DATA TYPES
*************************************************************************/

/* A TF binding site */
typedef struct s_site                  
{
  long Position1;
  long Position2;
  float Score; 
  char Name[MAXSTRING];
} site;

typedef struct s_map             
{
  char Locusname[MAXSTRING];
  site* bindingSites;              
  long  nBindingSites;                  
} map;

typedef struct s_coordinates             
{
  long x;
  long y;
} coordinates;

typedef struct s_path            
{
  coordinates v[MAXCOLISIONS];
  long nElements;
} path;

/* Matrices of pointers: P and S */
typedef struct s_packAlignment
{
  float*** S;
  path*** P;
} packAlignment;

typedef struct s_lnode* plnode;          
typedef struct s_lnode
{
  long x;
  long y;
  plnode next;
  plnode last;
} lnode; 

typedef struct s_touchList                  
{
  plnode first;
  long nElems; 
} touchList;

typedef struct s_account               
{
  time_t tStart;
  /* Number of optimal meta-alignments */
  long nAlignments;
  /* Number of matches between one list and the other */
  long nMatches;
  /* Number of accesses to the touch list */
  long nAccesses;
} account;


/*************************************************************************
(C). IMPORTED HEADERS
*************************************************************************/

void printError(char *s);

void printMess(char* s);

void printRes(char* s);

void readargv (int argc,char* argv[],
			   char* MapFile1, 
			   char* MapFile2);

map* RequestMemoryMap();

account* RequestMemoryAccounting();

lnode* RequestMemoryNode(long i, long j);

float* RequestOneSimilarityElement(float initValue);

touchList* RequestMemoryList();

path* RequestOnePathElement();

account* InitAcc();

void OutputHeader();

void OutputAccounting();

long ReadSitesGFF (char *FileName, map* mp);

packAlignment* RequestMemoryAlignment();

void  ComputeSimilarity(float alpha,
						float lambda, 
						float mu,
						map* map1, 
						map* map2,
						packAlignment* alignment,
						touchList* list);

void OutputAlignment(map* map1,map* map2,packAlignment* alignment);

void OutputReportAlignment(long i);

void OutputEndAlignment();

void OutputStartAlignment();

