/*************************************************************************
*                                                                        *
*   Module: ComputeSimilarity                                            *
*                                                                        *
*   Filling the similarity matrix (pairwise alignment)                   *
*                                                                        *
*************************************************************************/

#include "meta.h"

extern account *m;
extern int ALL;

void PrintList(touchList* l,packAlignment* alignment)
{
  lnode* p;
  long i;

  i = 0;
  p = l->first;
  while (i<l->nElems)
    {
      if (p->x != GHOST && p->y != GHOST)
	fprintf(stderr,"(%ld %ld:%.2f)\t|\t",p->x,p->y,*alignment->S[p->x][p->y]);
      else
	fprintf(stderr,"(%ld %ld:-)\t|\t",p->x,p->y);
      p=p->next;
      i++;
    }
  fprintf(stderr,"\n\n");
}

void UpdatePath(float newSim,
		float oldSim,
		path*** P,
		long i,
		long j,
		long oldi,
		long oldj)
{
  /* Current alignment is better than previous ones */
  if (newSim > oldSim)
    {
      P[i][j]->v[0].x = oldi;
      P[i][j]->v[0].y = oldj;
      P[i][j]->nElements = 1;
    }
  else
    {
      /* Colision (==): add the new point in the matrix */
      /* if it is necessary to display all of the optimal alignments */
      if (ALL)
	{
	  P[i][j]->v[P[i][j]->nElements].x = oldi;
	  P[i][j]->v[P[i][j]->nElements].y = oldj;
	  P[i][j]->nElements++; 
	  
	  if (P[i][j]->nElements == MAXCOLISIONS)
            printError("Too much colisions: increase the value MAXCOLISIONS");
	}
    }
}

float ComputeInitialSimilarity(site* a,
			       site* b,
			       long i,
			       long j,
			       float alpha,
			       float lambda,
			       float mu)
{
  float value;
  float alphaValue;
  float lambdaPenalty;
  float muPenalty;

  alphaValue = alpha *	(a->Score + b->Score);
  lambdaPenalty = lambda * (i + j - 2);
  muPenalty = mu*abs(a->Position1 - b->Position1);
  
  value = alphaValue - lambdaPenalty - muPenalty; 

  return value;
}

void  ComputeSimilarity(float alpha, 
			float lambda, 
			float mu,
			map* map1, map* map2,
			packAlignment* alignment,
			touchList* list)
{
  long  i,j,iPrev,jPrev;
  float tmpValue;
  float initSimilarity;
  float currentSim;
  float alphaValue;
  float lambdaPenalty;
  float muPenalty;
  int stop;
  lnode*  pAux;

  lnode*  newNode;
  lnode*  pointer;
  lnode*  next;
  long index;
  
  OutputStartAlignment();  
  
  /* Dynamic programming recurrence */
  for(i=0; i < map1->nBindingSites; i++)
    {
      if (!(i % 100))
	OutputReportAlignment(i);
      
      for(j=0; j < map2->nBindingSites; j++)
	{
	  /* MATCH between both lists */
	  if (!strcmp((map1->bindingSites+i)->Name,(map2->bindingSites+j)->Name))
	    {		
	      /* Compute initial similarity value: alignment first element */
	      initSimilarity = ComputeInitialSimilarity((map1->bindingSites+i),
							(map2->bindingSites+j),
							i,j,
							alpha,lambda,mu);

	      alignment->S[i][j] = (float*) RequestOneSimilarityElement(initSimilarity);
	      alignment->P[i][j] = (path*) RequestOnePathElement();
	      
	      /* Matching score */
	      alphaValue = alpha *
		((map1->bindingSites+i)->Score + (map2->bindingSites+j)->Score);
	      
	      /* Searching the best previous element to connect with current match */
	      /* Last element is a fixed GHOST element: he will never be evaluated */
	      index = 0;
	      pAux = list->first;	   
	      m->nAccesses++;
	      stop = ((index  == list->nElems - 1) || 
		      (*alignment->S[pAux->x][pAux->y] + alphaValue < *alignment->S[i][j])); 
	      while (!stop)
		{
		  iPrev = pAux->x;
		  jPrev = pAux->y;
		  
		  /* Do they overlap or not? */
		  if (((map1->bindingSites+i)->Position1 
		       - 
		       (map1->bindingSites+iPrev)->Position2 >= MINDIST) && 
		      ((map2->bindingSites+j)->Position1
		       -
		       (map2->bindingSites+jPrev)->Position2 >= MINDIST))
		    {              
		      tmpValue = (map1->bindingSites+i)->Position1 
			- (map1->bindingSites+iPrev)->Position1 
			- (map2->bindingSites+j)->Position1 
			+ (map2->bindingSites+jPrev)->Position1;  
		      
		      lambdaPenalty = lambda *((i-iPrev) + (j-jPrev) - 2);
		      muPenalty = mu*abs(tmpValue);
		      
		      currentSim =
			alphaValue + 
			*alignment->S[iPrev][jPrev] 
			- lambdaPenalty 
			- muPenalty; 
		      
		      /* Updating the best value */
		      if (currentSim >= *alignment->S[i][j])
			{
			  /* Updating the path */
			  UpdatePath(currentSim,
				     *alignment->S[i][j],
				     alignment->P,
				     i,j,iPrev,jPrev);
			  
			  /* Updating the best minimum distance */
			  *alignment->S[i][j] = currentSim;
			  
			}
		    }
		  
		  /* Next combination in the touch list, if any */           
		  index++; 
		  pAux = pAux->next;
		  m->nAccesses++;
			      
		  /* The while is always broken before the ghost element */
		  stop = ((index == list->nElems - 1) 
			  || (*alignment->S[pAux->x][pAux->y] + alphaValue < *alignment->S[i][j])); 
		}
	  
	      /* The reference point is pAux: current element MUST go before */
	      /* If index == list->nElems - 1, then pAux is the GHOST */
	      /* Otherwise, the element must go before pAux and possibly others */
	      /* Creating the new node */
	      newNode = RequestMemoryNode(i,j);
	      
	      /* The list must be always ordered (>) */
	      /* Using backward node pointers */
	      pointer = pAux->last;
	      m->nAccesses++;

	      /* Searching the proper position for the new node */
	      /* First candidate to have the current node afterwards */
	      stop = ((pointer == NULL) || (*alignment->S[pointer->x][pointer->y] >= *alignment->S[i][j]));
	      while(!stop)
		{
		  pointer = pointer->last;
		  m->nAccesses++;	      

		  stop = ((pointer == NULL) ||
			  (*alignment->S[pointer->x][pointer->y] >= *alignment->S[i][j]));
		}

	      /* Insertion of the new node into the list */
	      /* Place it at the beginning */
	      if (pointer == NULL)
		{
		  newNode->next = list->first;
		  list->first->last = newNode;
		  list->first = newNode;
		}
	      else
		{
		  /* Insert the element right there, after that node */
		  /* This node will never be the GHOST */
		  next = pointer->next;

		  pointer->next = newNode;
		  newNode->last= pointer;
		  newNode->next = next;
		  next->last = newNode;
		}

	      list->nElems++;
	      
	      /* Accounting stats */
	      m->nMatches++;
	    }
	}
    }
  
  OutputEndAlignment();
}

