/*************************************************************************
*                                                                        *
*   Module: Output                                                       *
*                                                                        *
*   Management: displaying results                                       *
*************************************************************************/

#include "meta.h"

extern int VRB;
extern int GFF;  

extern float alpha;
extern float lambda;
extern float mu;

extern account *m;

/* Printing messages (information) */
void printMess(char* s)
{
  if (VRB)
    fprintf (stderr, "> %s\n",s);
}

/* Printing stats and results (information) */
void printRes(char* s)
{
  if (VRB)
    fprintf (stderr, "\t%s\n",s);
}

/* Printing error messages */
void printError(char* s)
{
  fprintf(stderr,"Error: %s\n",s);
  exit(1);
}

/* Output: header for results displayed immediately */
void OutputHeader()
{
  char* s;

  /* 1. Extract the starting time to display */
  s = ctime(&m->tStart);
  printf("# %s\n",VERSION);
  printf("# date %s",s);  
}

void OutputReportAlignment(long i)
{
  if (VRB)
    fprintf(stderr,"[%ld]...",i);
}

void OutputStartAlignment()
{
  if (VRB)
    fprintf(stderr,"[STARTING]\n");
} 

void OutputEndAlignment()
{
  if (VRB)
    fprintf(stderr,"[END]\n\n");
}

/* Output the $ token */
void OutputToken(map* map1,
		 map* map2,
		 long i,
		 coordinates aln[])
{
  if (GFF)
    {
      /* First input sequence: */
      printf("%s\t%s\t%s\t.\t.\t.\t.\n",
	     map1->Locusname,
	     VERSION,
	     (map1->bindingSites+aln[i].x)->Name);
      
      /* Second input sequence: */
      printf("%s\t%s\t%s\t.\t.\t.\t.\t.\n",
	     map2->Locusname,
	     VERSION,
	     (map2->bindingSites+aln[i].y)->Name); 
	}
  else
    {
      /* Display the symbol */
      printf("%s:\n",
	     (map1->bindingSites+aln[i].x)->Name);
    }  
}

void OutputMatch(map* map1,
		 map* map2,
		 long i,
		 coordinates aln[])
{
  if (GFF)
    {
      /* First input sequence: */
      printf("%s\t%s\t%s\t%ld\t%ld\t%f\t.\t.\n",
	     map1->Locusname,
	     VERSION,
	     (map1->bindingSites+aln[i].x)->Name,
	     (map1->bindingSites+aln[i].x)->Position1,
	     (map1->bindingSites+aln[i].x)->Position2,
	     (map1->bindingSites+aln[i].x)->Score);
      
      /* Second input sequence: */
      printf("%s\t%s\t%s\t%ld\t%ld\t%f\t.\t.\n",
	     map2->Locusname,
	     VERSION,
	     (map2->bindingSites+aln[i].y)->Name,
	     (map2->bindingSites+aln[i].y)->Position1,
	     (map2->bindingSites+aln[i].y)->Position2,
	     (map2->bindingSites+aln[i].y)->Score); 
    }
  else
    {
      /* Display the symbol and positions in both sequences */
      printf("%s:\t\t %ld-%ld\t :\t %ld-%ld \t%.2f-%.2f\n",
	     (map1->bindingSites+aln[i].x)->Name,
	     (map1->bindingSites+aln[i].x)->Position1,
	     (map1->bindingSites+aln[i].x)->Position2,
	     (map2->bindingSites+aln[i].y)->Position1,
	     (map2->bindingSites+aln[i].y)->Position2,
	     (map1->bindingSites+aln[i].x)->Score,
	     (map2->bindingSites+aln[i].y)->Score);
    }  
}

void RecoverAlignment(map* map1,
		      map* map2,
		      packAlignment* alignment,
		      long currentX,
		      long currentY,
		      coordinates aln[],
		      long nElements)
{
  long newX;
  long newY;
  long i;
  
  /* A. Direct call */
  if (currentX == BEGIN && currentY == BEGIN)
    {
      /* END OF CHAIN: recover the whole list of matches */
      printf("### Best meta-alignment contains %ld elements\n",nElements-1);  
      
      /* Print the matches series */
      for(i=nElements-1; i >=1; i--)
	OutputMatch(map1,map2,i,aln);
      
      /* Multiple optimal solutions: option x (ALL) */
      m->nAlignments++;
    }
  else
    {
      /* B. Recursive call */
      /* insert the current element into the alignment */
      aln[nElements].x = currentX;
      aln[nElements].y = currentY;
      
      /* For each sibling do the recursive call */
      /* Multiple optimal solutions: option x (ALL) */
      for(i=0; i < alignment->P[currentX][currentY]->nElements; i++)
	{
	  /* Update the new coordinates in the matrix */
	  newX = alignment->P[currentX][currentY]->v[i].x;
	  newY = alignment->P[currentX][currentY]->v[i].y;
	  
	  /* Next call */
	  RecoverAlignment(map1,map2,alignment,newX,newY,aln,nElements+1);
	}
    }
}

void OutputAlignment(map* map1, map* map2,packAlignment* alignment)
{
  float numerator,denominator;

  coordinates aln[MAXPAIRS];
  
  printf("# MAP1 %s - Length = %ld elems\n",
	 map1->Locusname,
	 map1->nBindingSites-NTOKENS);
  
  printf("# MAP2 %s - Length = %ld elems\n",
	 map2->Locusname,
	 map2->nBindingSites-NTOKENS);
  
  printf("# ALPHA = %.2f, LAMBDA = %.2f, MU = %.2f\n", 
	 alpha,
	 lambda,
	 mu); 

  printf("# Maximum similarity: %.2f\n", 
	 *alignment->S[map1->nBindingSites-1][map2->nBindingSites-1]); 

  numerator = m->nMatches-NTOKENS;
  denominator = (map1->nBindingSites - NTOKENS) * (map2->nBindingSites - NTOKENS);

  printf("# SimMatrix: %.0f matches / %.0f positions (%.2f %%)\n", 
	 numerator,
	 denominator,
	 (100 * numerator)/denominator); 
 
  /* Recursive call to produce the alignment */
  RecoverAlignment(map1,map2,alignment,
		   map1->nBindingSites-1,
		   map2->nBindingSites-1,aln,0);
}

/* Computing running time by using accounting information */
void OutputAccounting()
{
  time_t tEnd;
  int t;
  float caux;
  char mess[MAXSTRING];
  
  printMess("Accounting stats:");
  
  sprintf(mess,"%ld\t\tmatches between both lists",
	  m->nMatches-NTOKENS);
  printRes(mess);
  
  sprintf(mess,"%ld\t\toptimal alignments",
	  m->nAlignments);
  printRes(mess);
  
  sprintf(mess,"%ld\t\taccesses to the touch List",
	  m->nAccesses);
  printRes(mess);
  
  /* Final time */
  /* Real time */
  (void) time(&tEnd);
  /* CPU time */
  caux = (float)clock() / (float)CLOCKS_PER_SEC;
  
  t = (int) tEnd - m->tStart;
  
  /* Correction */
  if (t < caux)
    t++;

  printRes("__________________________________________________________________________\n");
  
  sprintf(mess,"CPU time: \t%.3f secs",caux);
  printRes(mess);
  
  sprintf(mess,"Total time: \t%d secs(%.2f mins)\n\n",t,(float)t/MINUTE);
  printRes(mess);
}
