/*************************************************************************
*                                                                        *
*   Module: ReadSitesGFF                                                 *
*                                                                        *
*   Reading binding sites (GFF format) from file                         *
*                                                                        *
*************************************************************************/

#include "meta.h"

/* GFF format: tab "\t" is the field separator and # for comments */
/* Name  Source  Type  Begin  End  Score  Strand  Frame  [group] */
long ReadSitesGFF (char *FileName, map* mp)
{
  /* File handle */
  FILE *file;
  
  /* Final number of sites loaded from file  */
  long i;
  
  /* Split every input line into several tokens (gff records) */
  char line[MAXLINE];
  char lineCopy[MAXLINE];
  char *line1;
  char *line2;
  char *line3;
  char *line4;
  char *line5;
  char *line6;
  char *line7;
  char *line8;
  char *line9;
  
  /* Control of good sorting property: starting position, increasing */
  long lastSite;
  long currSite;

  char mess[MAXSTRING];
  

  /* 0. Open GFF file to read the information */
  if ((file=fopen(FileName, "r"))==NULL)
    printError("The GFF file can not be opened to read");
  
  /* 1. Reset counters */
  i = 0;
  lastSite = -INFI;
  
  /* 2. Read while there are exons left in the input file */
  while(fgets(line,MAXLINE,file)!=NULL)
    {
      /* 2.a. Comment or empty line: forget it */
      if (line[0]=='#' || line[0]=='\n')
	printMess("Skipping comment line in MAP file");
      else
	{
	  /* 2.b. Processing site */
	  /* Backup copy of the line to show error messages */
	  strcpy(lineCopy,line);
	  
	  /* Extracting GFF features */
	  line1 = (char *) strtok(line,"\t");
	  line2 = (char *) strtok(NULL,"\t");
	  line3 = (char *) strtok(NULL,"\t"); 
	  line4 = (char *) strtok(NULL,"\t");
	  line5 = (char *) strtok(NULL,"\t");
	  line6 = (char *) strtok(NULL,"\t");
	  line7 = (char *) strtok(NULL,"\t");
	  line8 = (char *) strtok(NULL,"\t");
	  line9 = (char *) strtok(NULL,"\n");
	  
	  /* There are 8 mandatory columns and the last one is optional */
	  if (line1 == NULL || line2 == NULL || line3 == NULL ||
	      line4 == NULL || line5 == NULL || line6 == NULL ||
	      line7 == NULL || line8 == NULL)
	    {
	      sprintf(mess,"Wrong GFF format (number of records):\n-->%s\n",lineCopy);
	      printError(mess);
	    }
	  
	  /* 1/2. Save the sequence (map) id */
	  strcpy(mp->Locusname,line1);
	  
	  /* 3. Binding site feature: name of the TF matrix */
	  if (sscanf(line3,"%s",(mp->bindingSites+i)->Name) != 1)
	    {
	      sprintf(mess,"Wrong GFF format (feature):\n-->%s\n",lineCopy);
	      printError(mess);
	    }
	  
	  /* 4. Starting position */
	  if (sscanf(line4,"%ld",&((mp->bindingSites+i)->Position1)) != 1)
	    {
	      sprintf(mess,"Wrong GFF format (starting position):\n-->%s\n",lineCopy);
	      printError(mess);
	    }
	  
	  /* 5. Finishing position */
	  if (sscanf(line5,"%ld",&((mp->bindingSites+i)->Position2)) != 1)
	    {
	      sprintf(mess,"Wrong GFF format (finishing position):\n-->%s\n",lineCopy);
	      printError(mess);
	    }
	  
	  /* 6. Score = float value or '.'(infinitum) */
	  if (sscanf(line6,"%f",&((mp->bindingSites+i)->Score)) != 1)
	    {
	      sprintf(mess,"Wrong GFF format in annotations (score):\n-->%s\n",lineCopy);
	      printError(mess);
	    }
	  
	  /* Strand, frame and group discarded */
	  /* 7. Strand (reading sense) [+|-] */
	  /* 8. Frame = integer or '.' */
	  /* 9. Group: optional, string */
	  
	  /* (B). Checking the sorting property */
	  currSite = (mp->bindingSites + i)->Position1;
	  if (lastSite > currSite)
	    {
	      sprintf(mess,"Wrong order in (starting position %ld):\n-->%s\n",
		      lastSite,
		      lineCopy);  
	      printError(mess);  
	    }
	  else
	    lastSite = currSite;
	  
	  /* Next record */
	  i++;
	  
	  if (i > MAXSITES)
	    printError("Too many binding sites: increase MAXSITES value (meta.h)");
	  
	} /* End of checkpoint for comments (#) */
    } /* End of while (input sites) */
  fclose(file);

  /* Creating the $ ghost site */
  strcpy((mp->bindingSites+i)->Name,TOKEN);
  (mp->bindingSites+i)->Position1 = INFI;
  (mp->bindingSites+i)->Position2 = INFI;
  (mp->bindingSites+i)->Score = NOSCORE;
  i++; 

  /* Return the number of created sites */
  return(i);
}

