/*************************************************************************
*                                                                        *
*   Module: RequestMemory                                                *
*                                                                        *
*   Request memory to operating system for general data structures       *
*                                                                        *
*************************************************************************/

#include "meta.h"

/* Allocating accounting data structure in memory */
account* RequestMemoryAccounting()
{
  account* m; 

  if ((m = (account *) malloc(sizeof(account))) == NULL)
	printError("Not enough memory: account");

  return(m);
}

/* Map of predicted binding sites */
map* RequestMemoryMap()
{
  map* p;

  /* Allocating memory for the general structure */
  if ((p = 
       (struct s_map *) malloc(sizeof(struct s_map))) == NULL)
    printError("Not enough memory: (map) general structure");  
  
  /* Allocating memory for the array of binding sites */
  if ((p->bindingSites = 
       (struct s_site *) calloc(MAXSITES, sizeof(struct s_site))) == NULL)
    printError("Not enough memory: (map) array of binding sites");
  
  p->nBindingSites = 0;

  return(p);
}

/* Memory for a vector of coordinates or path element */
path* RequestOnePathElement()
{
  path* p;
  
  if ((p =  (path *) malloc(sizeof(path))) == NULL)
	printError("Not enough memory for a path-element"); 

  /* Initializing: acting as a first element of a possible alignment */
  p->v[0].x = BEGIN;
  p->v[0].y = BEGIN;
  
  p->nElements = 1;
   
  return(p);
}

/* Memory for the current similarity value */
float* RequestOneSimilarityElement(float initValue)
{
  float* p;
  
  if ((p =  (float *) malloc(sizeof(float))) == NULL)
    printError("Not enough memory for a sim-element"); 

  /* Initializing: acting as a first element of a possible alignment */
  *p = initValue;

  return(p);
}

/* Map of predicted binding sites */
packAlignment* RequestMemoryAlignment()
{
  packAlignment* p;
  long i;

  /* Allocating memory for the general structure */
  if ((p = 
       (struct s_packAlignment *) malloc(sizeof(struct s_packAlignment))) == NULL)
    printError("Not enough memory: (packAlignment) general structure");  
  
  if ((p->S = 
       (float ***) calloc(MAXSITES, sizeof(float** ))) == NULL)
    printError("Not enough memory: (packAlignment) Sim matrix (dim 1)");  
  
  for(i=0; i < MAXSITES; i++)
    {
      /* allocating rows */
      if ((p->S[i] = 
	   (float **) calloc(MAXSITES, sizeof(float*))) == NULL)
	printError("Not enough memory: (packAlignment) Sim matrix (dim 2)");
    }

  if ((p->P = 
       (path ***) calloc(MAXSITES, sizeof(path** ))) == NULL)
    printError("Not enough memory: (packAlignment) P matrix (dim 1)");  
  
  for(i=0; i < MAXSITES; i++)
    { 
      /* allocating rows */ 
      if ((p->P[i] = 
	   (path **) calloc(MAXSITES, sizeof(path*))) == NULL)
	printError("Not enough memory: (packAlignment) P matrix (dim 2)");
    }  
  
  return(p);
}

/* Memory for a node associated to a non empty position in the S matrix */
lnode* RequestMemoryNode(long i, long j)
{
  lnode* n;

  if ((n = (lnode *) malloc(sizeof(lnode))) == NULL)
	printError("Not enough memory: new lnode");
  
  n->x = i;
  n->y = j;
  n->next = NULL;
  n->last = NULL;

  return(n);
}

/* List of non-empty positions into the sparse matrix S[i][j] */
touchList* RequestMemoryList()
{
  touchList* l;

  if ((l = (touchList *) malloc(sizeof(touchList))) == NULL)
	printError("Not enough memory: touchList");
  
  /* GHOST Insertion */
  l->first = RequestMemoryNode(GHOST,GHOST);
  l->nElems = 1;

  return(l);
}

