/*************************************************************************
*                                                                        *
*   Module: meta                                                         *
*                                                                        *
*   meta main program                                                    *
*                                                                        *
*************************************************************************/

#include "meta.h"

/* meta setup flags */
int
  /* Verbose flag (memory/processing information) */
  VRB=0,
  /* Flag to display the complete set of optimal alignments */
  ALL=0,
  /* Flag to print GFF output */
  GFF=0;  

/* Alignment control parameters */
float alpha, lambda, mu;

/* Accounting time and results */
account *m;

/************************************************************************
                            meta MAIN program
************************************************************************/

int main (int argc, char *argv[])
{
  /* Data structures for the TF binding sites */
  map* map1;
  map* map2;
  packAlignment* alignment;
  touchList* list;

  /* Input GFF Filenames */
  char   MapFile1[FILENAMELENGTH],
   MapFile2[FILENAMELENGTH];
  
  char mess[MAXSTRING];

  /** 0. Starting and reading options and maps **/
  
  /* 0.a. Initializing stats and time counters */
  m = (account*)InitAcc();  
  
  /* 0.b. Read setup options */
  readargv(argc,argv,MapFile1,MapFile2);
  printRes("\n\n\t\t\t** Executing meta 1.1 2005 **\n\n");
  
  /* Output header information */
  OutputHeader();

  /** 1. Allocating main geneid data structures **/
  printMess("Request Memory to Operating System\n");
  
  /* 1.a. Mandatory META data structures */
  map1      = (map*)           RequestMemoryMap();
  map2      = (map*)           RequestMemoryMap();  
  alignment = (packAlignment*) RequestMemoryAlignment();
  list      = (touchList*)     RequestMemoryList();

  /* 2. Reading both maps: <bs,pos1,pos2> */

  /* 2.a. MAP1: */
  printMess("Reading MAP1");

  map1->nBindingSites =
    ReadSitesGFF(MapFile1, map1);
  if (map1->nBindingSites == NTOKENS)
    printError("MAP1: no elements have been provided");
  else
    {
      sprintf(mess,"MAP1: %s %ld binding sites\n", 
	      map1->Locusname,
	      map1->nBindingSites-NTOKENS); 
      printMess(mess);
    }  

  /* 2.b. MAP2: */
  printMess("Reading MAP2");
  
  map2->nBindingSites =
	ReadSitesGFF(MapFile2, map2);
  if (map2->nBindingSites == NTOKENS)
    printError("MAP2: no elements have been provided");
  else
    {
      sprintf(mess,"MAP2: %s %ld binding sites\n", 
	      map2->Locusname,
	      map2->nBindingSites-NTOKENS); 
      printMess(mess);
    }

  /* 3. Generation of the meta-alignment */
  printMess("Computing the meta-alignment");
  
  sprintf(mess,"ALPHA = %.2f, LAMBDA = %.2f, MU = %.2f\n", 
	  alpha,
	  lambda,
	  mu); 
  printMess(mess);
  
  ComputeSimilarity(alpha,lambda,mu,map1,map2,alignment,list);

  sprintf(mess,"# Maximum similarity: %.2f\n", 
	  *alignment->S[map1->nBindingSites-1][map2->nBindingSites-1]); 
  printMess(mess);

  /* 4. Output results */
  printMess("Recovering the best meta-alignment...");
  OutputAlignment(map1,map2,alignment);

  OutputAccounting();  
  
  /* 5. The end */
  exit(0);
  return(0);
}
