/*************************************************************************
*                                                                        *
*   Module: readargv                                                     *
*                                                                        *
*   Read set up options and filenames from user input                    *
*                                                                        *
*************************************************************************/

#include "meta.h"

/* meta.c external vars */
extern int  VRB, GFF, ALL;

/* Alignment control parameters */
extern float lambda, mu, alpha;

/* required by getopts */
extern char* optarg;
extern int optind;

char* USAGE="NAME\n\tmeta - a program to produce meta-alignments of sequences of TF binding sites\nSYNOPSIS\n\tmeta\t[-alm] [-g] [-v] [-h]\t<map1.gff> <map2.gff>\nRELEASE\n\tmeta_v1.1\n";

void printHelp()
{
  printf(USAGE);

  printf ("OPTIONS\n");
  
  printf("\t-a: Set value for the alpha penalty\n");
  printf("\t-l: Set value for the lambda penalty\n");
  printf("\t-m: Set value for the mu penalty\n");
  
  printf("\t-g: Print GFF-formatted output\n"); 

  printf("\t-x: Display the complete set of optimal solutions\n");  

  printf("\t-v: Verbose. Display info messages\n");
  printf("\t-h: Show this help\n");

  printf ("AUTHORS\n");
  printf("\tmeta v1.1 has been developed by Enrique Blanco, Xavier Messeguer and Roderic Guigo.\n");

  printf("\n\n\n");
}

void readargv (int argc,
	       char* argv[],
	       char* MapFile1, 
	       char* MapFile2) 
{
  int c;
  char mess[MAXSTRING];
  
  /* Default values for Alpha, Lambda and Mu penalties */
  alpha = ALPHA_DEFAULT;
  lambda = LAMBDA_DEFAULT;
  mu = MU_DEFAULT;
  
  /* Reading setup options */
  while ((c = getopt(argc,argv,"a:l:m:gxvh")) != -1)
    switch(c)
      {
      case 'a': alpha = atof(optarg);
	break;
      case 'l': lambda = atof(optarg);
	break;
      case 'm': mu = atof(optarg);
	break;
      case 'x': ALL++;
	break;
      case 'g': GFF++;
	break;
      case 'v': VRB++;
	break;
      case 'h': printHelp();
	exit(0);
	break;
      }
  
  /* Setup Errors (b): Wrong number of filenames */
  /* Read the names of the two input gff files */
  if (optind < argc)
    {
      strcpy(MapFile1,argv[optind]);
      optind++;
      if (optind < argc)
		{ 
		  strcpy(MapFile2,argv[optind]);
		  optind++;
		  if (optind < argc)
		   {
			 sprintf(mess,"More than two input GFF files have been provided while two are just required\n%s",USAGE);
			 printError(mess);
		   }
		}
	  else
		{
          sprintf(mess,"Only one input filename has been provided but two are required\n%s",USAGE);
		  printError(mess);
		}
    }
  else
	{
	  sprintf(mess,"Two input GFF files are required\n%s",USAGE);
	  printError(mess);
	}
}
